; FILE:	tinytimer.asm					W. Waetzig	26.11.2009
;		AVR Studio V. 4.14 - AVRISP mkll
;
; Programmable short-time (seconds resolution) timer with display
;	- setting of hours, minutes and seconds
;	- the external load is switched by a relay
;
; * Microprocessor		ATMEL ATtiny 2313
;	4.9152 MHz quartz-clock 
; * LCD-display HMC16223 (Hitachi standard)
;	with 2*16 characters in 4-bit data-mode
;
; Connections to the ATtiny 2313
;	PB0 (output) - LCD-E4
;	PB1 (output) - LCD-D5
;	PB2 (output) - LCD-D6
;	PB3 (output) - LCD-D7
;	PB4 (output) - LCD-RS (reg-select)
;	PB5 (output) - LCD-EN (strobe) 	(MOSI)
;	PB6 (output) - LED green		(MISO)
;	PB7 (output)					(SCK)
;	PA2 (input)  - RESET
;	PD0 (output) 
;	PD1 (output) 
;	PD2 (input-pullup)  - switch START
;	PD3 (input-pullup)  - switch STOP/MENU
;	PD4 (input-pullup)  - switch SET '+'
;	PD5 (input-pullup)  - switch SET '-'
;	PD6 (output) - drive relay
;
;**************************************************************************
;
; Functional description of the timer
;
; The microprocessor works as a one-second counter and counts down 
; the counting-time in hours, minutes and seconds set by the switches.
; The following information is displayed:
;		PRESET  0h 0m 0s   
;		COUNTER STOPPED
; in counting-mode (example):
;		PRESET  1h10m 8s 
;		COUNT   0h 9m59s
; alternative display:
;		PRESET  1:10:08 
;		COUNT   1:09:59 
; Four pushbuttons control the timer:
;	- switch_start:	start the timer
;	- switch_stop:	stop the timer / select MENUE
;	- switch_plus:	increment the selected value
;	- switch_minus:	decrement the selected value
;
; The stop-switch steps through the menue:
;	MENUE-1	SET HOURS   00
;	MENUE-2	SET MINUTES 00
;	MENUE-3	SET SECONDS 00
;	MENUE-4	SET DISPMODE 0
; The switches PLUS and MINUS increment/decrement the value;
; both together clear the value.
;
; At start timer the following values are saved in EEPROM:
;	EEPROM(0) = sethour
;	EEPROM(1) = setmins
;	EEPROM(2) = setsecs
;	EEPROM(3) = dispmode
;
; FUSES:	EXT.	0xFF	- brown out det. off, no CKDIV8
;			HIGH	0xDF	- ext. crystal > 3 MHz
;			LOW		0xFD	- 64ms start up
;
;**************************************************************************
.nolist
.include	<tn2313def.inc>	; ATtiny 2313 
.list
.listmac
;************************************* MACROS *****************************
;
.macro	loadz	; @0:ADDRESS from program-memory
	ldi	Zl,low(@0*2)
	ldi	Zh,high(@0*2)
.endmacro
;
.macro	loadx	; @0:ADDRESS from SRAM	
	ldi	Xl,low(@0)
	ldi	Xh,high(@0)
.endmacro
;
;********************************* VARIABLES ******************************
.DSEG
LCDcounter:	.byte 1	; count characters for LCD-line
LCDvalue:	.byte 1	; LCD output-value
;
tim1ovfl:	.byte 1 ; timer-1 overflow-indicator
timerrun:	.byte 1 ; timer is running
dispmode:	.byte 1 ; display-mode 0/1
menuflag:	.byte 1 ; flag for menue
;
sethour:	.byte 1 ; preset hours	--- must be in this sequence
setmins:	.byte 1 ; preset minutes
setsecs:	.byte 1 ; preset seconds
;
cnthour:	.byte 1 ; count hours	--- must be in this sequence
cntmins:	.byte 1 ; count minutes
cntsecs:	.byte 1 ; count seconds
;
;******************************** INTERRUPT VECTORS ***********************
.CSEG
.ORG	$00
	rjmp	reset
	reti
	reti
	reti
	reti
	rjmp	timer1_OVF
	reti
	reti
	reti
	reti
	reti
	reti
	reti
	reti
	reti
	reti
	reti
	reti
	reti
;
;********************************** TABLES ********************************
;
; set pins on PORTD/PIND switches
.equ	swstart=2
.equ	swstop=3
.equ	swplus=4
.equ	swminus=5
.equ	outrelay=6
; set pins PORTB display
.equ	outlrs=4
.equ	outena=5
.equ	outled=6
;
; Message strings, zero-terminated
;
MSG1:		.DB "TinyTimer 0911  ", 0,0
MSG2:		.DB " hours/mins/secs", 0,0
MSGsethour:	.DB "SET HOURS   ", 0,0
MSGsetmin:	.DB "SET MINUTES ", 0,0
MSGsetsec:	.DB "SET SECONDS ", 0,0
MSGsetdsm:	.DB "SET DISPMODE ", 0
MSGpreset:	.DB "PRESET ", 0
MSGcount:	.DB "COUNT  ", 0
MSGstop:	.DB "COUNTER STOPPED ", 0,0
MSGerror:	.DB "ERROR no PRESET ", 0,0
;
;***************************** TIMER-INTERRUPT ***************************** 
;
; TIMER-1 interrupt on overflow every second 
timer1_OVF:
	push	r16
	in	r16,SREG
	push	r16
; set overflow-indicator
	ldi	r16,high(65536-4800)
	out	TCNT1H,r16
	ldi	r16,low(65536-4800)
	out	TCNT1L,r16
	ser	r16
	sts	tim1ovfl,r16	; ==> set 1-second flag
	sbi	PINB,7		; === TEST: toggle PB7
; restore and return
	pop	r16
	out	SREG,r16
	pop	r16
	reti
;
;******************************* RESET *************************************
;	
; Initialise the stack-pointer	
reset:
	ldi	r16,low(RAMEND)
	out	SPL,r16
;	ldi	r16,high(RAMEND)
;	out	SPH,r16					
; initialize PORTB - PB6=LED green, PB7 = test
	ldi	r16,0xC0	;  bit 7..6 = output
	out	DDRB,r16	;PORTB I/O => LCD
	ldi	r16,0x00
	out	PORTB,r16	; clear PORTB
; initialize PORTD
	ldi	r24,(1<<outrelay)
	out	DDRD,r24	; =>PORTD-output
	ldi	r24,(1<<swstart)|(1<<swstop)|(1<<swplus)|(1<<swminus)
	out	PORTD,r24	; =>PORTD pullup bits
; TIMER-1 initialization, quartz = 4.9152MHz, scaler: 1024 => 4800
	ldi	r16,0
	out	TCCR1A,r16	; normal operation
	out	TCCR1C,r16	; normal operation
	rcall	timer1_stop
	in	r16,TIMSK
	ori	r16,(1<<TOIE1)	; enable TOIE1 interrupt
	out	TIMSK,r16
; clear timer-values
	rcall	timer_clear
	rcall	timer1_stop
; initialize the LCD
	rcall	LCD_init
	rjmp	Bstart
;
;************************** SUBROUTINES ***********************
; stop TIMER-1
timer1_stop:
	ldi	r16,0x00
	out	TCCR1B,r16	; stop timer1
	clr	r16
	sts	tim1ovfl,r16	; clear overflow-indicator
	sts	timerrun,r16	; stop timer running
	ret
;**************************************************************
; start TIMER-1
timer1_start:
	ldi	r16,high(65536-4800)
	out	TCNT1H,r16
	ldi	r16,low(65536-4800)
	out	TCNT1L,r16
	ldi	r16,0x05
	out	TCCR1B,r16	; (CS12,CS10): clock / 1024
	clr	r16
	sts	tim1ovfl,r16	; clear overflow-inidicator
	ser	r16
	sts	timerrun,r16	; start timer running
	ret
;**************************************************************
; clear timer-values
timer_clear:
	clr	r16
	sts	sethour,r16
	sts	setmins,r16
	sts	setsecs,r16
	sts	cnthour,r16
	sts	cntmins,r16
	sts	cntsecs,r16
	sts	timerrun,r16
	ret
;**************************************************************
; Delay100s		4.9152 Mhz-Quartz
;**************************************************************
Delay100us:
	ldi	r17,98
delay1us:	; 98*1.017 microsec
	nop
	nop
	dec	r17
	brne	delay1us	; => 5 clock-cycles => 1.017 microsec
	ret	
;**************************************************************
; Delay 10 ms		<r16> * 10ms
;**************************************************************
Delay10ms:
	ldi	r18,100
delay10x:
	rcall	delay100us
	dec	r18
	brne	delay10x
	dec	r16
	brne	Delay10ms
	ret
;**************************************************************
; Delay 1 ms		<r16> * 1ms		
;**************************************************************
Delay1ms:
	ldi	r18,10
delay1x:
	rcall	delay100us
	dec	r18
	brne	delay1x
	dec	r16
	brne	Delay1ms
	ret
;**************************************************************
; Initialize the LCD using PORTB 5..0
;	PB0 (output) - LCD-D4
;	PB1 (output) - LCD-D5
;	PB2 (output) - LCD-D6
;	PB3 (output) - LCD-D7
;	PB4 (output) - LCD-RS (reg-select)
;	PB5 (output) - LCD-EN (strobe) 
;**************************************************************
LCD_init:
; initialize PORTB - LCD driver, PB0..PB5
	in	r16,DDRB
	ori	r16,0x3F	; bit 5..0 = output
	out	DDRB,r16	; PORTB I/O => LCD
	ldi	r16,0x00
	out	PORTB,r16	; clear PORTB
; 250 ms delay of Vdd rise for LCD
	ldi	r16,25
	rcall	Delay10mS
; init-commands for the LDC (HITACHI standard)
	ldi	r16,2
	rcall	LCD_initcom
	ldi	r16,2
	rcall	LCD_initcom
	ldi	r16,2			;4 bit operation
	rcall	LCD_initcom
	ldi	r16,0b00100000		;4 bit operation
	rcall	LCD_setcom
 	ldi	r16,0b00101000		;2 lines, 5x8 display
	rcall	LCD_setcom
	ldi	r16,0b00001000		;display off, cursor off, blink off
	rcall	LCD_setcom
	ldi	r16,0b00001100		;display on,cursor off, blink off
	rcall	LCD_setcom
	ldi	r16,0b00000110		;cursor postion moves to the right
	rcall	LCD_setcom
	ldi	r16,0b00000010		;DDRam-adress=0, (most left position)
	rcall	LCD_setcom
	ldi	r16,0b00000001		;clear the Display
	rcall	LCD_setcom
	ret
;**************************************************************
; LCD_wport	Write 4 data-bits to the LCD
;**************************************************************
LCD_wport:
	sbrc	r16,4	; RS-bit
	sbi	PORTB,outlrs
	sbrs	r16,4	; RS-bit
	cbi	PORTB,outlrs
	andi	r16,0b00001111
	in	r17,PORTB
	andi	r17,0b11110000
	or	r17,r16
	out	PORTB,r17	; output 4-bit data
	sbi	PORTB,outena		;enable hi
	rcall	delay100uS
	cbi	PORTB,outena		;enable lo
	rcall	delay100uS
	ret
;***************************************************************
; LCD_dispchar	Displays a character in <r16>, LCD: 2*16 char.
; LCD_dispdig	Displays a digit in <r16>
;	Writes in 4 bit mode,  MSB is first; character in <r16>
;	<r16>: bit0-3=(D4-D7), bit4=RS (1=data)
;***************************************************************
LCD_dispdig:
	andi	r16,0x0F
	ori	r16,'0'
LCD_dispchar:
	sts	LCDvalue,r16
; check if LCD-counter = 17
	lds	r16,LCDcounter
	inc	r16
	sts	LCDcounter,r16
	cpi	r16,17
	brne	LCD_dispnext
	rcall	LCD_setline2
; write MSB
LCD_dispnext:
	lds	r16,LCDvalue
	swap	r16			
	andi	r16,0x0F
	ori	r16,(1<<4)	;bit4 =  RS (1=data)
	rcall	LCD_wport	; upper nibble
; write LSB
	lds	r16,LCDvalue
	andi	r16,0x0F
	ori	r16,(1<<4)	;RS = 1
	rcall	LCD_wport	; lower nibble
	rcall	delay100uS
	ret
;***************************************************************
; LCD_dspaces	Displays <r16> spaces
;***************************************************************
LCD_dspaces:
	mov	r24,r16
LCD_dspace0:
	ldi	r16,' '
	rcall	LCD_dispchar
	dec	r24
	brne	LCD_dspace0
	ret
;***************************************************************
; LCD_setline1		set cursor in LCD to pos 0 in line1
; LCD_setline2		set cursor in LCD to begin of Line2
; LCD_setcom	<r16> send command to LCD 
; LCD_initcom	<r16> Initializes the LCD in 4-bit-mode
;	Writes in 4-bit-mode,  MSB is first, command-code in <r16>
;	<r16>: bit0-3=(D4-D7), bit4=RS (0=command)
;***************************************************************
LCD_setline1:
	ldi	r16,0b00000010
	clr	r24	
	rjmp	LCD_setcom0		;cursor 0, line1, cnt=0
LCD_setline2:
	ldi	r16,0b11000000		;set to pos. 0 in line 2, cnt=17
	ldi	r24,17
LCD_setcom0:
	sts	LCDcounter,r24		; set LCD char. counter
LCD_setcom:
; write MSB
	mov	r24,r16
	swap	r16			
	andi	r16,0x0F	; bit4=0
	rcall	LCD_wport	; upper nibble
; write LSB
	mov	r16,r24
LCD_initcom:
	andi	r16,0x0F	; bit4=0
	rcall	LCD_wport	; lower nibble
	ldi	r16,5
	rcall	Delay1ms	
	ret
;***************************************************************
; LCD_gotoxy	set cursor in LCD in line x=0..1 to pos y=0..15 
;		x=upper nibble, x=lower nibble in <r16>	
;***************************************************************
LCD_gotoxy:
	mov	r17,r16
	com	r17
	andi	r17,0x10
	lsl	r17
	lsl	r17		; line# = bit6
	ori	r17,0x80	; bit7=pos-command
	andi	r16,0x0F	; bit0..3=char-pos
	or	r16,r17
	rcall	LCD_setcom
	ret
;***************************************************************
; LCD_dispmsg	Displays a message terminated by zero
;	Line must be selected by LCD_setline1 or LCD_setline2 routine call
;	pointer to message address in <ZH,ZL>
;***************************************************************
LCD_dispmsg:
	lpm	r16,Z+
	tst	r16
	breq	LCD_dispmxx
	rcall	LCD_dispchar
	rjmp	LCD_dispmsg
LCD_dispmxx:
	ret	
;***************************************************************
; write a byte into EEPROM, address in <r17>, byte in <r16>
;***************************************************************
eeprom_write:
	rcall	eeprom_addr
	out	EEDR,r16	; EEPROM-data
	sbi	EECR,EEMPE	; 1=EEMPE enable writing
	sbi	EECR,EEPE	; start EEPROM-write
	ret
;***************************************************************
; read a byte from EEPROM, address in <r17>, byte in <r16>
;***************************************************************
eeprom_read:
	rcall	eeprom_addr
	sbi	EECR,EERE	; start EEPROM-read
	in	r16,EEDR	; get byte from data-register
	ret
;***************************************************************
eeprom_addr:
	sbic	EECR,EEPE	; wait for completion of previous write
	rjmp	eeprom_addr
	out	EEARL,r17	; EEPROM-address low
;	clr	r0
;	out	EEARH,r0	; EEPROM-address high
	ret	
;***************************************************************
; convnum:	convert and display 2-digit number in <r16>
;			suppress leading zero
;***************************************************************
convnum:
	mov	r20,r16
	clr	r16
convn02:
	inc	r16
	subi	r20,10
	brpl	convn02
	subi	r20,-10
	dec	r16
	brne	convn03
	ldi	r16,' '
	rjmp	convn04
convn03:	
	ori	r16,'0'
convn04:
	rcall	LCD_dispchar	; 1-st digit or space
	mov	r16,r20
	rcall	LCD_dispdig	; 2-nd digit
	ret
;***************************************************************
; convnum0:	convert and display 2-digit number in <r16>
;			with leading zero
;***************************************************************
convnum0:
	mov	r20,r16
	clr	r16
convn01:
	inc	r16
	subi	r20,10
	brpl	convn01
	subi	r20,-10
	dec	r16
	rcall	LCD_dispdig	; 1-st digit
	mov	r16,r20
	rcall	LCD_dispdig	; 2-nd digit
	ret
;***************************************************************
; display timer-values (dispmode); X points to 1st value
disp_timer:
; check for dispmode
	lds	r16,dispmode
	sbrs	r16,0
	rjmp	disp_tima
; normal display		..h..m..s
	ld	r16,X+		; display hours
	rcall	convnum
	ldi	r16,'h'
	rcall	LCD_dispchar
	ld	r16,X+		; display minutes
	rcall	convnum
	ldi	r16,'m'
	rcall	LCD_dispchar
	ld	r16,X+		; displayseconds
	rcall	convnum
	ldi	r16,'s'
	rcall	LCD_dispchar
	rjmp	disp_timx
; alternative display	hh:mm:ss
disp_tima:
	ld	r16,X+		; display hours
	rcall	convnum
	ldi	r16,':'
	rcall	LCD_dispchar
	ld	r16,X+		; display minutes
	rcall	convnum0
	ldi	r16,':'
	rcall	LCD_dispchar
	ld	r16,X+		; display seconds
	rcall	convnum0
	ldi	r16,' '
	rcall	LCD_dispchar
disp_timx:
	ret
;***************************************************************
; display preset-time (dispmode)
disp_preset:
	rcall	LCD_setline1
	loadz	MSGpreset	;	"PRESET "
	rcall	LCD_dispmsg
	loadx	sethour		; X points to first preset value
	rcall	disp_timer	; display timer-values
	ret
;***************************************************************
; display time counted (dispmode)
disp_count:
	rcall	LCD_setline2
	lds	r16,timerrun	; is the timer running?
	sbrc	r16,0
	rjmp	dtimcntc
	loadz	MSGstop		; "COUNTER STOPPED "
	rcall	LCD_dispmsg
	rjmp	dtimcntx
dtimcntc:
	loadz	MSGcount		;	"COUNT  "
	rcall	LCD_dispmsg
	loadx	cnthour		; X points to first counter value
	rcall	disp_timer	; display timer-values	
dtimcntx:
	ret
;***************************************************************
; test if switch is ON
; input <r16>: (1<<switch); return <r16>: 0=no switch, 1=switch 
test_switchon:
	in	r15,PIND	; read switches
	com	r15
	and	r16,r15
	breq	testsw00
	ser	r16
testsw00:
	cpi	r16,0
	ret
;***************************************************************
; wait until switch is OFF
; input <r16>: (1<<switch), wait 10 ms
wait_switchoff:
	mov	r14,r16
waitsw01:
	ldi	r16,10
	rcall	delay1ms
	in	r15,PIND	; read switches
	and	r15,r14		; check if switch is OFF
	cp	r15,r14
	brne	waitsw01
	ldi	r16,10
	rcall	delay1ms
	ret
;***************************************************************
; decrement timer: seconds/minutes/hours
; return <r16>: 0=continue; 1=stop timer
dectimer:
	lds	r16,cntsecs		; decrement seconds
	dec	r16
	sts	cntsecs,r16
	brpl	dectimxx
	ldi	r16,59
	sts	cntsecs,r16
	lds	r16,cntmins		; decrement minutes
	dec	r16
	sts	cntmins,r16
	brpl	dectimxx
	ldi	r16,59
	sts	cntmins,r16
	lds	r16,cnthour		; decrement hours
	dec	r16
	sts	cnthour,r16
	brpl	dectimxx
	ser	r16
	sbrs	r16,0
dectimxx:
	clr	r16
	cpi	r16,0		; set return-value
	ret
;***************************************************************
; start timer
;  copy PRESET hours/minutes/seconds into COUNTER and into EEPROM,
;  switch-on RELAY and LED, start TIMER-1
;***************************************************************
start_timer:
	sbi	PORTB,outled	; switch on green LED
	sbi	PORTD,outrelay	; switch on RELAY
; write preset-times into EEPROM
	lds	r16,sethour	
	sts	cnthour,r16		; copy hours
	ldi	r17,0
	rcall	eeprom_write	; save cnthour
	lds	r16,setmins	
	sts	cntmins,r16		; copy minutes
	ldi	r17,1
	rcall	eeprom_write	; save cntmins
	lds	r16,setsecs	
	sts	cntsecs,r16		; copy seconds
	ldi	r17,2
	rcall	eeprom_write	; save cntsecs
	lds	r16,dispmode
	ldi	r17,3
	rcall	eeprom_write	; save dispmode
	rcall	timer1_start	; start timer running
	ret
;***************************************************************
; stop timer
stop_timer:
	rcall	timer1_stop
	clr	r16
	sts	cnthour,r16
	sts	cntmins,r16
	sts	cntsecs,r16
	cbi	PORTB,outled	; switch off green LED
	cbi	PORTD,outrelay	; switch off RELAY
	ret
;***************************************************************
; toggle display mode
toggle_disp:
	lds	r16,dispmode
	com	r16
	sts	dispmode,r16
	ldi	r17,3
	rcall	eeprom_write	; save dispmode
 	ret
;***************************************************************
; increment hours
inc_hours:
	lds	r16,sethour
	cpi r16,99
	breq	inchour1
	inc	r16
	rjmp	inchour2
inchour1:
	clr	r16
inchour2:
	sts	sethour,r16
	ret
;***************************************************************
; decrement hours
dec_hours:
	lds	r16,sethour
	cpi r16,0
	breq	dechour1
	dec	r16
	rjmp	dechour2
dechour1:
	ldi	r16,99
dechour2:
	sts	sethour,r16
	ret
;***************************************************************
; increment minutes
inc_mins:
	lds	r16,setmins
	cpi	r16,59
	breq	incmins1
	inc	r16
	rjmp	incmins2
incmins1:
	clr	r16
incmins2:
	sts	setmins,r16
	ret
;***************************************************************
; decrement minutes
dec_mins:
	lds	r16,setmins
	cpi	r16,0
	breq	decmins1
	dec	r16
	rjmp	decmins2
decmins1:
	ldi	r16,59
decmins2:
	sts	setmins,r16
	ret
;***************************************************************
; increment seconds
inc_secs:
	lds	r16,setsecs
	cpi	r16,59
	breq	incsecs1
	inc	r16
	rjmp	incsecs2
incsecs1:
	clr	r16
incsecs2:
	sts	setsecs,r16
	ret
;***************************************************************
; decrement seconds
dec_secs:
	lds	r16,setsecs
	cpi	r16,0
	breq	decsecs1
	dec	r16
	rjmp	decsecs2
decsecs1:
	ldi	r16,59
decsecs2:
	sts	setsecs,r16
	ret
;***************************************************************
; display hours/minutes/seconds/dispmode
disp_hmsd:
; branch to menues
	lds	r16,menuflag		; get menue-flag
	cpi	r16,1
	breq	disp_menu1
	cpi	r16,2
	breq	disp_menu2
	cpi	r16,3
	breq	disp_menu3
; disp_menu0
	rcall	LCD_setline2
	loadz	MSGsetdsm
	rcall	LCD_dispmsg	; "SET DISPMODE "
	lds	r16,dispmode
	andi	r16,1
	rjmp	disp_exit
disp_menu1:
	rcall	LCD_setline2
	loadz	MSGsethour
	rcall	LCD_dispmsg	; "SET HOURS   "
	lds	r16,sethour	; display hours
	rjmp	disp_exit
disp_menu2:
	rcall	LCD_setline2
	loadz	MSGsetmin
	rcall	LCD_dispmsg	; "SET MINUTES "
	lds	r16,setmins	; display minutes
	rjmp	disp_exit
disp_menu3:
	rcall	LCD_setline2
	loadz	MSGsetsec
	rcall	LCD_dispmsg	; "SET SECONDS "
	lds	r16,setsecs	; display seconds
disp_exit:
	rcall	convnum	
	ldi	r16,2
	rcall	LCD_dspaces
	ret
;***************************************************************
; increment hours/minutes/seconds/dispmode
incr_hmsd:
; branch to menues
	lds	r16,menuflag		; get menue-flag
	cpi	r16,1
	breq	incr_menu1
	cpi	r16,2
	breq	incr_menu2
	cpi	r16,3
	breq	incr_menu3
; incr_menu0
	lds	r16,dispmode
	com	r16
	sts	dispmode,r16
	rjmp	incr_exit
incr_menu1:
	rcall	inc_hours
	rjmp	incr_exit
incr_menu2:
	rcall	inc_mins
	rjmp	incr_exit
incr_menu3:
	rcall	inc_secs
incr_exit:
	ret
;***************************************************************
; decrement hours/minutes/seconds/dispmode
decr_hmsd:
; branch to menues
	lds	r16,menuflag		; get menue-flag
	cpi	r16,1
	breq	decr_menu1
	cpi	r16,2
	breq	decr_menu2
	cpi	r16,3
	breq	decr_menu3
; disp_menu0
	lds	r16,dispmode
	com	r16
	sts	dispmode,r16
	rjmp	decr_exit
decr_menu1:
	rcall	dec_hours
	rjmp	decr_exit
decr_menu2:
	rcall	dec_mins
	rjmp	decr_exit
decr_menu3:
	rcall	dec_secs
decr_exit:
	ret
;***************************************************************
; clear hours/minutes/seconds/dispmode
clrv_hmsd:
; branch to menues
	lds	r16,menuflag		; get menue-flag
	cpi	r16,1
	breq	clrv_menu1
	cpi	r16,2
	breq	clrv_menu2
	cpi	r16,3
	breq	clrv_menu3
	rjmp	clrv_exit
clrv_menu1:
	ldi	r16,0
	sts	sethour,r16
	rjmp	clrv_exit
clrv_menu2:
	ldi	r16,0
	sts	setmins,r16
	rjmp	clrv_exit
clrv_menu3:
	ldi	r16,0
	sts	setsecs,r16
clrv_exit:
	ret
;
;******************** MAIN PROGRAM *****************************
;
Bstart:
; read last preset-values from EEPROM
	ldi	r17,0
	rcall	eeprom_read	; read sethour
	cpi	r16,0xFF
	breq	read_eep1
	sts	sethour,r16
read_eep1:
	ldi	r17,1
	rcall	eeprom_read	; read setmins
	cpi	r16,0xFF
	breq	read_eep2
	sts	setmins,r16
read_eep2:
	ldi	r17,2
	rcall	eeprom_read	; read setsecs
	cpi	r16,0xFF
	breq	read_eep3
	sts	setsecs,r16
read_eep3:
	ldi	r17,3
	rcall	eeprom_read	; read dispmode
	sts	dispmode,r16
; display message
	ldi	r16,0
	sts	menuflag,r16	; clear menue-flag
	loadz	MSG1	; "TinyTimer 0911  "
	rcall	LCD_setline1
	rcall	LCD_dispmsg
	rcall	LCD_setline2
	loadz	MSG2	; " hours/mins/secs"
	rcall	LCD_dispmsg
	ldi	r16,200
	rcall	delay10ms
	sei			; ===> enable interrupt
;
;********************* MAIN LOOP *******************************
main_display:
	rcall	disp_preset		; display preset values
	rcall	disp_count		; display timer-values
Main_Loop:
	ldi	r16,20
	rcall	delay10ms
; check if timer is running
	lds	r16,timerrun
	cpi	r16,0
	breq	main_switch	
main_ovfl:
	lds	r16,tim1ovfl	; wait for timer-overflow
	cpi	r16,0
	brne	main_timer
; check if stop-switch or plus-switch or minus-switch
	ldi	r16,(1<<swstop)|(1<<swplus)|(1<<swminus)
	rcall	test_switchon
	breq	main_ovfl
	rjmp	main_stop
; timer-Overflow, display timer preset
main_timer:
	clr	r16
	sts	tim1ovfl,r16	; clear overflow
	rcall	dectimer	; decrement timer-values (sec/min/hrs)
	sbrs	r16,0
	rjmp	main_display
; end of timer-counting
main_stop:
	rcall	stop_timer
	ldi	r16,25
	rcall	delay10ms
	rjmp	main_display
; error, no preset-time given
main_error:
	rcall	LCD_setline2
	loadz	MSGerror	; "ERROR no PRESET"
	rcall	LCD_dispmsg
	ldi	r16,200
	rcall	delay10ms
	rjmp	main_stop
;
; =============== timer is NOT running: read switches =========
main_switch:
; >>>>>> check for start-switch
	ldi	r16,(1<<swstart)
	rcall	test_switchon	; start-switch
	breq	main_swstop
	ldi	r16,(1<<swstart)|(1<<swstop)|(1<<swplus)|(1<<swminus)
	rcall	wait_switchoff	; wait until all switches are released
; check, if preset-time == 0
	lds	r17,sethour
	lds	r16,setmins
	add	r17,r16
	lds	r16,setsecs
	add	r17,r16
	breq	main_error
; start timer, set timerrun-flag; decrement by 1 second
	rcall	start_timer
	rjmp	main_timer
; >>>>>> check for stop/menu-switch
main_swstop:
	ldi	r16,(1<<swstop)
	rcall	test_switchon	; stop-switch 
	breq	main_switch
	ldi	r16,(1<<swstop)
	rcall	wait_switchoff	; wait until stop-switch is released
	lds	r16,menuflag
	inc	r16
	andi	r16,3
	sts	menuflag,r16		; increment menue-flag: 0..3
; ================ menue: hours-minutes-seconds-dispmode ============
main_menu00:
	rcall	disp_preset	; display preset-values
	rcall	disp_hmsd	; ======> display test and values
; check plus/minus switches
main_menurs:
	ldi	r16,10
	rcall	delay10ms
	ldi	r16,(1<<swplus)
	rcall	test_switchon	; switch-PLUS
	brne	main_menu10
	ldi	r16,(1<<swminus)
	rcall	test_switchon	; switch-MINUS
	brne	main_menu11
; exit if menu-switch is pressed
	ldi	r16,(1<<swstop)
	rcall	test_switchon	; stop-switch 
	breq	main_menurs		; => read switches
	ldi	r16,(1<<swstop)
	rcall	wait_switchoff	; wait until stop-switch is released
	rjmp	main_display
main_menu10:
	ldi	r16,(1<<swminus)
	rcall	test_switchon	; switch-minus also pressed?
	brne	main_menu12
	ldi	r16,(1<<swplus)
	rcall	wait_switchoff	; wait until plus-switch is released
	rcall	incr_hmsd	; ======> increment hours/minutes/seconds/dispmode
	rjmp	main_menu00
main_menu11:
	ldi	r16,(1<<swplus)
	rcall	test_switchon	; switch-plus also pressed?
	brne	main_menu12
	ldi	r16,(1<<swminus)
	rcall	wait_switchoff	; wait until minus-switch is released
	rcall	decr_hmsd	; ======> decrement hours/minutes/seconds/dispmode
	rjmp	main_menu00
main_menu12:
	ldi	r16,(1<<swplus)|(1<<swminus)
	rcall	wait_switchoff	; wait until plus/minus-switch is released
	rcall	clrv_hmsd	; ======> clear hours/minutes/seconds
	rjmp	main_menu00
;
;******************************* END ************************************

	
